public class DotMakerHashtable {

  private final HashMap hashMap;
  private final StringBuilder b = new StringBuilder();
  private int nextId = 0;
  private float width;
  
  public static String makeDot(HashMap hashMap) {
    return new DotMakerHashtable(hashMap).getDot();
  }

  public static String makeDot(HashMap hashMap, float width) {
    return new DotMakerHashtable(hashMap, width).getDot();
  }
  
  private DotMakerHashtable(HashMap hashMap) {
    this(hashMap, 2.9f);
  }

  private DotMakerHashtable(HashMap hashMap, float width) {
    this.hashMap = hashMap;
    this.width = width;
  }

  private String getDot() {
    b.append("digraph MyGraph {\n");
    b.append("nodesep=.05;\nrankdir=LR;\n");
    table();
    chains();
    b.append("}");
    return b.toString();
  }

  private void table() {
    b.append("node [shape=record,width=.1,height=.1]\n\n");
    b.append(String.format("tab[width=%.1f,", width));
    b.append("label=\"");
    for (HashMap.Node n: hashMap.tab) {
      if (n != null) {
        if (n.next != null) {
          b.append(String.format("<f%d>", ++nextId));
        }
        b.append(escapeSpaces(n.key.toString()));
      } else {
        b.append('-');
      }
      b.append('|');
    }
    if(hashMap.tab.length > 0) {
      b.delete(b.length()-1, b.length());
    }
    b.append("\",");
    b.append("]");
    b.append(";\n\n");
  }
  
  private void chains() {
    nextId = 0;
    b.append("node [shape=box,fixedsize=true,labeloc=\"l\",");
    b.append(String.format("width=%.1f,height=.28]\n", width));
    for (HashMap.Node n: hashMap.tab) {
      if (n != null && n.next != null) {
        HashMap.Node p = n.next;
        while (p != null) {
          b.append(String.format("node%d[label=\"", ++nextId));
          b.append(escapeSpaces(p.key.toString()));
          b.append("\"];\n");
          if (p == n.next) {
            b.append(String.format(
              "tab:f%d:e->node%d:_:w;\n", nextId, nextId));
          } else {
            b.append(String.format(
              "node%d:_:e->node%d:_:w;\n", nextId-1, nextId));
          }
          p = p.next;
        }
      }
    }
  }
  
  private String escapeSpaces(String s) {
    StringBuilder b = new StringBuilder(s);
    for (int i=b.length()-1; i >= 0; i--) {
      if (b.charAt(i) == ' ') {
        b.insert(i, '\\');
      }
    }
    return b.toString();
  }
}